#!/usr/bin/env python3
from typing import List

from orger import Mirror
from orger.common import dt_heading, error
from orger.inorganic import node, link

import my.pinboard as pinboard


class PinboardView(Mirror):
    def get_items(self) -> Mirror.Results:
        items: List[pinboard.Bookmark] = []
        for b in pinboard.bookmarks():
            if isinstance(b, Exception):
                yield error(b)
            else:
                items.append(b)

        for b in sorted(
            items,
            # need to sort by some other property since all initial exports have the same timestamp
            # doesn't look like there is any sort of bookmark id for pinboard
            key=lambda b: (b.created, b.url),
        ):
            yield node(
                heading=dt_heading(b.created, link(title=b.title, url=b.url)),
                body=b.description,
                tags=b.tags,
            )


test = PinboardView.make_test(
    heading='Cartesian Closed Comic #21',
    contains='doctorwho',  # todo predicate?
)

if __name__ == '__main__':
    PinboardView.main()


# todo need to use hpi module install my.pinbpoar



# this is a hack to monkey patch pytest so it handles tests inside namespace packages without __init__.py properly
# without it, pytest can't discover the package root for some reason
# also see https://github.com/karlicoss/pytest_namespace_pkgs for more

import os
import pathlib
from typing import Optional

import _pytest.main
import _pytest.pathlib

# we consider all dirs in repo/ to be namespace packages
root_dir = pathlib.Path(__file__).absolute().parent.resolve() / 'src'
assert root_dir.exists(), root_dir

# TODO assert it contains package name?? maybe get it via setuptools..

namespace_pkg_dirs = [str(d) for d in root_dir.iterdir() if d.is_dir()]

# resolve_package_path is called from _pytest.pathlib.import_path
# takes a full abs path to the test file and needs to return the path to the 'root' package on the filesystem
resolve_pkg_path_orig = _pytest.pathlib.resolve_package_path
def resolve_package_path(path: pathlib.Path) -> Optional[pathlib.Path]:
    result = path  # search from the test file upwards
    for parent in result.parents:
        if str(parent) in namespace_pkg_dirs:
            return parent
    if os.name == 'nt':
        # ??? for some reason on windows it is trying to call this against conftest? but not on linux/osx
        if path.name == 'conftest.py':
            return resolve_pkg_path_orig(path)
    raise RuntimeError("Couldn't determine path for ", path)
_pytest.pathlib.resolve_package_path = resolve_package_path


# without patching, the orig function returns just a package name for some reason
# (I think it's used as a sort of fallback)
# so we need to point it at the absolute path properly
# not sure what are the consequences.. maybe it wouldn't be able to run against installed packages? not sure..
search_pypath_orig = _pytest.main.search_pypath
def search_pypath(module_name: str) -> str:
    mpath = root_dir / module_name.replace('.', os.sep)
    if not mpath.is_dir():
        mpath = mpath.with_suffix('.py')
        assert mpath.exists(), mpath  # just in case
    return str(mpath)
_pytest.main.search_pypath = search_pypath



#!/usr/bin/env python3
from orger import Queue
from orger.inorganic import node, link
from orger.common import todo

from my.instapaper import pages, is_todo


class IpTodos(Queue):
    def get_items(self) -> Queue.Results:
        for page in pages():
            bm = page.bookmark
            for hl in page.highlights:
                if not is_todo(hl):
                    continue
                yield hl.hid, todo(
                    # todo dt_heading? not sure. Maybe this should be configurable
                    dt=hl.dt,
                    heading=link(title="X", url=hl.instapaper_link) + ' ' + hl.text,
                    tags=['ip2org'],
                    body=f'{hl.note}\nfrom {link(title=bm.title, url=bm.url)}',
                )


if __name__ == '__main__':
    IpTodos.main()


# TODO move error handling to the base class
# TODO add some test?



#!/usr/bin/env python3
from orger import Mirror
from orger.inorganic import node, link, docview_link, literal
from orger.common import dt_heading, error

from datetime import datetime


class PdfView(Mirror):
    def get_items(self) -> Mirror.Results:
        import my.pdfs as pdfs

        for pdf in sorted(
                pdfs.annotated_pdfs(),
                key=lambda p: datetime.min if isinstance(p, Exception) or p.created is None else p.created.replace(tzinfo=None),
        ):
            if isinstance(pdf, Exception):
                yield error(pdf)
                continue

            def chit(pdf: pdfs.Pdf):
                for a in pdf.annotations:
                    parts = []
                    highlight = (a.highlight or '').strip()
                    author    = (a.author    or '').strip()
                    comment   = (a.comment   or '').strip()
                    if highlight:
                        parts.append(literal(highlight))
                    if author:
                        parts.append(f'by {author}')
                    if comment:
                        parts.append(comment)
                    body = '\n'.join(parts)
                    page1 = a.page + 1
                    page_link = docview_link(path=pdf.path, title=f'page {page1}', page1=page1)
                    yield node(
                        dt_heading(a.created, page_link),
                        body=body,
                    )

            pdf_link = docview_link(path=pdf.path, title=str(pdf.path))  # todo would be nice to extract metadata for title
            yield node(
                dt_heading(pdf.created, pdf_link),
                children=list(chit(pdf))
            )


if __name__ == '__main__':
    # import logging
    # from orger.common import setup_logger
    # setup_logger(pdfs.logger, level=logging.DEBUG)
    PdfView.main()



#!/usr/bin/env python3
from orger import Mirror
from orger.inorganic import node
from orger.common import dt_heading, error


import my.config
# todo later, use .all?
import my.tinder.android as tinder

class TinderView(Mirror):
    def get_items(self) -> Mirror.Results:
        res = None
        for i in tinder.match2messages():
            assert res is None, res
            if isinstance(i, Exception):
                yield error(i)
            else:
                res = i
        assert res is not None

        for match, messages in sorted(res.items(), key=lambda p: p[0].when):
            yield node(
                dt_heading(match.when, match.person.name),
                children=[node(
                    dt_heading(m.sent, f'{m.from_.name}: {m.text}'),
                ) for m in messages]
            )



if __name__ == '__main__':
    TinderView.main()
